"""
Module for HTTP Load Balancers
https://docs.cloud.f5.com/docs-v2/api/views-http-loadbalancer
"""

from uplink import Consumer, Path, Body, get, post, put, delete, json  # pylint: disable=unused-import
from . import helper

@helper.common_decorators
class HTTPLoadBalancer(Consumer):
    """
    Class for HTTP Load Balancer API.
    """

    def __init__(self, session):
        """
        Initialize the HTTPLoadBalancer Consumer.
        :param session: Session object with tenant URL and auth.
        """
        super().__init__(base_url=session._tenant_url, client=session._session)

    @get('/api/config/namespaces/{namespace}/http_loadbalancers')
    def list(self, namespace: Path):
        """
        List all HTTP Load Balancers in a namespace.
        """

    @get('/api/config/namespaces/{namespace}/http_loadbalancers/{name}')
    def get(self, namespace: Path, name: Path):
        """
        Get details of an HTTP Load Balancer.
        """

    @json
    @post('/api/config/namespaces/{namespace}/http_loadbalancers')
    def create(self, payload: Body, namespace: Path):
        """
        Create an HTTP Load Balancer.
        Use create_payload() to build the request body.
        """

    @json
    @put('/api/config/namespaces/{namespace}/http_loadbalancers/{name}')
    def replace(self, payload: Body, namespace: Path, name: Path):
        """
        Replace an HTTP Load Balancer.
        Use create_payload() to build the request body.
        """

    @delete('/api/config/namespaces/{namespace}/http_loadbalancers/{name}')
    def delete(self, namespace: Path, name: Path):
        """
        Delete an HTTP Load Balancer.
        """

    @staticmethod
    def create_payload(
        name: str,
        namespace: str,
        listeners: list,
        origin_pools: list,
        policies: list = None,
        tls_config: dict = None,
        use_mtls: bool = False,
        description: str = "",
        labels: dict = None,
    ):
        """
        Create a payload for an HTTP Load Balancer.

        :param name: Name of the HTTP load balancer.
        :param namespace: Namespace for the load balancer.
        :param listeners: List of listener configurations (use build_listener()).
        :param origin_pools: List of linked origin pools.
        :param policies: List of traffic policies (optional).
        :param tls_config: TLS configuration options (optional).
        :param use_mtls: Whether to use mutual TLS (optional).
        :param description: Description of the load balancer.
        :param labels: Labels to tag the load balancer.
        :return: Dictionary payload for the API request.
        """
        if not name or not namespace:
            raise ValueError("Both 'name' and 'namespace' are required.")

        if not isinstance(listeners, list) or not listeners:
            raise ValueError("'listeners' must be a non-empty list.")

        if not isinstance(origin_pools, list) or not origin_pools:
            raise ValueError("'origin_pools' must be a non-empty list.")

        if labels is None:
            labels = {}
        if policies is None:
            policies = []

        payload = {
            "metadata": {
                "name": name,
                "namespace": namespace,
                "description": description,
                "labels": labels,
                "annotations": {},
            },
            "spec": {
                "listeners": listeners,
                "origin_pools": origin_pools,
                "policies": policies,
            },
        }

        if tls_config:
            payload["spec"]["tls_config"] = tls_config

        if use_mtls:
            payload["spec"]["use_mtls"] = use_mtls

        return payload

    @staticmethod
    def build_listener(
        name: str,
        port: int,
        protocol: str = "HTTP",
        tls_enabled: bool = False,
        use_mtls: bool = False,
        tls_config: dict = None,
    ):
        """
        Build an HTTP/HTTPS listener configuration for a Load Balancer.

        :param name: Unique name for the listener.
        :param port: Port number for the listener.
        :param protocol: Protocol type (default: "HTTP", can be "HTTPS").
        :param tls_enabled: Whether TLS is enabled (default: False).
        :param use_mtls: Whether to use mutual TLS (optional).
        :param tls_config: TLS configuration details (optional).
        :return: Dictionary representing an HTTP listener.
        """
        if protocol not in ["HTTP", "HTTPS"]:
            raise ValueError("Invalid protocol. Must be 'HTTP' or 'HTTPS'.")

        listener = {
            "name": name,
            "port": port,
            "protocol": protocol,
            "tls_enabled": tls_enabled,
        }

        if use_mtls:
            listener["use_mtls"] = use_mtls

        if tls_config:
            listener["tls_config"] = tls_config

        return listener

@helper.common_decorators
class TCPLoadBalancer(Consumer):
    """
    Class for TCP Load Balancer API.
    """

    def __init__(self, session):
        """
        Initialize the TCPLoadBalancer Consumer.
        :param session: Session object with tenant URL and auth.
        """
        super().__init__(base_url=session._tenant_url, client=session._session)

    @get('/api/config/namespaces/{namespace}/tcp_loadbalancers')
    def list(self, namespace: Path):
        """
        List all TCP Load Balancers in a namespace.
        """

    @get('/api/config/namespaces/{namespace}/tcp_loadbalancers/{name}')
    def get(self, namespace: Path, name: Path):
        """
        Get details of a TCP Load Balancer.
        """

    @json
    @post('/api/config/namespaces/{namespace}/tcp_loadbalancers')
    def create(self, payload: Body, namespace: Path):
        """
        Create a TCP Load Balancer.
        Use create_payload() to build Body.
        """

    @json
    @put('/api/config/namespaces/{namespace}/tcp_loadbalancers/{name}')
    def replace(self, payload: Body, namespace: Path, name: Path):
        """
        Replace a TCP Load Balancer.
        Use create_payload() to build Body.
        """

    @delete('/api/config/namespaces/{namespace}/tcp_loadbalancers/{name}')
    def delete(self, namespace: Path, name: Path):
        """
        Delete a TCP Load Balancer.
        """

    @staticmethod
    def create_payload(
        name: str,
        listeners: list,
        origin_pools: list,
        policies: list = None,
        description: str = "",
        labels: dict = None,
        annotations: dict = None,
        namespace: str = "system",
        disable: bool = False
    ):
        """
        Create a payload for TCP Load Balancer.

        :param name: Name of the TCP load balancer.
        :param listeners: List of listener configurations (use build_listener()).
        :param origin_pools: List of linked origin pools.
        :param policies: List of traffic policies (optional).
        :param description: Description of the load balancer.
        :param labels: Labels to tag the load balancer.
        :param annotations: Annotations for the load balancer.
        :param namespace: Namespace for the load balancer.
        :param disable: Whether the load balancer is administratively disabled.
        :return: Dictionary payload for the API request.
        """
        if labels is None:
            labels = {}
        if annotations is None:
            annotations = {}
        if policies is None:
            policies = []

        return {
            "metadata": {
                "name": name,
                "namespace": namespace,
                "description": description,
                "labels": labels,
                "annotations": annotations,
                "disable": disable
            },
            "spec": {
                "listeners": listeners,
                "origin_pools": origin_pools,
                "policies": policies
            }
        }

    @staticmethod
    def build_listener(
        name: str,
        port: int,
    ):
        """
        Build a TCP listener configuration for a Load Balancer.

        :param name: Unique name for the listener.
        :param port: Port number for the listener.
        :return: Dictionary representing a TCP listener.
        """
        return {
            "name": name,
            "port": port,
            "protocol": "TCP",
        }



"""
Module for Origin Pool
https://docs.cloud.f5.com/docs-v2/api/views-origin-pool
"""
from uplink import Consumer, Path, Body, get, post, put, delete, json #pylint: disable=unused-import
from . import helper

@helper.common_decorators
class OriginPool(Consumer):
    """
    Class for Origin Pool API.
    """

    def __init__(self, session):
        """
        Initialize the OriginPool Consumer.
        :param session: Session object with tenant URL and auth.
        """
        super().__init__(base_url=session._tenant_url, client=session._session)

    @get('/api/config/namespaces/{namespace}/origin_pools')
    def list(self, namespace: Path):
        """
        List all Origin Pools in a namespace.
        """

    @get('/api/config/namespaces/{namespace}/origin_pools/{name}')
    def get(self, namespace: Path, name: Path):
        """
        Get details of an Origin Pool.
        """

    @json
    @post('/api/config/namespaces/{namespace}/origin_pools')
    def create(self, payload: Body, namespace: Path):
        """
        Create an Origin Pool.
        Use create_payload() to build Body.
        """

    @json
    @put('/api/config/namespaces/{namespace}/origin_pools/{name}')
    def replace(self, payload: Body, namespace: Path, name: Path):
        """
        Replace an Origin Pool.
        Use create_payload() to build Body.
        """

    @delete('/api/config/namespaces/{namespace}/origin_pools/{name}')
    def delete(self, namespace: Path, name: Path):
        """
        Delete an Origin Pool.
        """

    @staticmethod
    def create_payload(
        name: str,
        namespace: str,
        origin_servers: list,
        port: int = 443,  # Explicitly adding port at the top level
        loadbalancer_algorithm: str = "LB_OVERRIDE",  # Changing to match working payload
        endpoint_selection: str = "LOCAL_PREFERRED",
        healthcheck: list = None,
        description: str = "",
        labels: dict = None,
        disable: bool = False,
        no_tls: dict = None,  # Adding `no_tls` parameter
        same_as_endpoint_port: dict = None,  # Keeping but making it optional
    ):
        """
        Construct the full payload for an Origin Pool.
        """
        if labels is None:
            labels = {}

        if healthcheck is None:
            healthcheck = []

        spec = {
            "origin_servers": [
                {k: v for k, v in server.items() if k != "name"}
                for server in origin_servers
            ],
            "port": port,  # Adding `port` explicitly
            "loadbalancer_algorithm": loadbalancer_algorithm,
            "endpoint_selection": endpoint_selection,
            "healthcheck": healthcheck,
        }

        # Add `same_as_endpoint_port` only if explicitly provided
        if same_as_endpoint_port is not None:
            spec["same_as_endpoint_port"] = same_as_endpoint_port

        # Ensure `no_tls` is always present as an empty object if not provided
        spec["no_tls"] = no_tls if no_tls is not None else {}

        return {
            "metadata": {
                "name": name,
                "namespace": namespace,
                "description": description,
                "labels": labels,
                "disable": disable,
            },
            "spec": spec,
        }

    @staticmethod
    def build_origin_server(
        name: str,
        port: int = 443,
        public_ip: str = None,
        public_name: str = None,
        private_ip: str = None,
        private_name: str = None,
        k8s_service: dict = None,
        custom_endpoint: dict = None,
        cbip_service: str = None,
        consul_service: dict = None,
        vn_private_ip: str = None,
        vn_private_name: str = None,
        refresh_interval: int = 300,
        site_locator: dict = None,
        labels: dict = None,
    ):
        """
        Build an origin server configuration for an Origin Pool.

        :param name: Unique name for the origin server.
        :param port: Port of the origin server (default: 443).
        :param public_ip: Public IP address of the origin server (optional).
        :param public_name: Public DNS name of the origin server (optional).
        :param private_ip: Private IP address of the origin server (optional).
        :param private_name: Private DNS name of the origin server (optional).
        :param k8s_service: Kubernetes service reference (optional, dict with 'service_name').
        :param custom_endpoint: Custom endpoint reference (optional, dict with 'name', 'namespace').
        :param cbip_service: Classic BIG-IP Virtual Server name (optional).
        :param consul_service: HashiCorp Consul service reference (optional, dict with 'service_name').
        :param vn_private_ip: Virtual Network IP address (optional).
        :param vn_private_name: Virtual Network DNS name (optional).
        :param refresh_interval: DNS resolution refresh interval (default: 300 seconds).
        :param site_locator: Site reference object (optional, dict with 'name', 'namespace').
        :param labels: Labels to tag the origin server (optional).
        :return: Dictionary representing an origin server.
        """
        if labels is None:
            labels = {}

        origin_server = {"name": name, "labels": labels, "port": port}

        if public_ip:
            origin_server["public_ip"] = {"ip": public_ip}

        if public_name:
            origin_server["public_name"] = {
                "dns_name": public_name,
                "refresh_interval": refresh_interval,
            }

        if private_ip:
            origin_server["private_ip"] = {"ip": private_ip}

        if private_name:
            origin_server["private_name"] = {
                "dns_name": private_name,
                "refresh_interval": refresh_interval,
            }

        if k8s_service:
            if not isinstance(k8s_service, dict) or "service_name" not in k8s_service:
                raise ValueError("'k8s_service' must be a dictionary with 'service_name'.")
            origin_server["k8s_service"] = k8s_service

        if custom_endpoint:
            if not isinstance(custom_endpoint, dict) or "name" not in custom_endpoint:
                raise ValueError("'custom_endpoint' must be a dictionary with 'name'.")
            origin_server["custom_endpoint_object"] = {"endpoint": custom_endpoint}

        if cbip_service:
            origin_server["cbip_service"] = {"service_name": cbip_service}

        if consul_service:
            if not isinstance(consul_service, dict) or "service_name" not in consul_service:
                raise ValueError("'consul_service' must be a dictionary with 'service_name'.")
            origin_server["consul_service"] = consul_service

        if vn_private_ip:
            origin_server["vn_private_ip"] = {"ip": vn_private_ip}

        if vn_private_name:
            origin_server["vn_private_name"] = {"dns_name": vn_private_name}

        if site_locator:
            if not isinstance(site_locator, dict) or "name" not in site_locator:
                raise ValueError("'site_locator' must be a dictionary with 'name'.")
            origin_server["site_locator"] = site_locator

        return origin_server
